"""
Tests of the model.Matrix33 class
"""

import math

import numpy as np

from flitter.model import Matrix33, Matrix44

from . import utils


class TestMatrix33(utils.TestCase):
    """
    Tests of the Matrix33 class

    Note that matrices are column-major as per OpenGL.
    """

    def test_construct(self):
        self.assertEqual(Matrix33(), [1, 0, 0, 0, 1, 0, 0, 0, 1])
        self.assertEqual(Matrix33(0), [0, 0, 0, 0, 0, 0, 0, 0, 0])
        self.assertEqual(Matrix33(1), [1, 0, 0, 0, 1, 0, 0, 0, 1])
        self.assertEqual(Matrix33(2), [2, 0, 0, 0, 2, 0, 0, 0, 2])
        self.assertEqual(Matrix33(range(9)), range(9))
        self.assertEqual(Matrix33([0, 1, 2, 3, 4, 5, 6, 7, 8]), range(9))
        self.assertEqual(Matrix33(np.arange(9)), range(9))
        self.assertRaises(ValueError, Matrix33, "Hello world!")
        self.assertRaises(ValueError, Matrix33, [1, 2, 3])
        self.assertRaises(ValueError, Matrix33, ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i'])
        self.assertRaises(ValueError, Matrix33, range(8))
        self.assertRaises(ValueError, Matrix33, np.arange(8))

    def test_copy(self):
        m1 = Matrix33(range(9))
        m2 = m1.copy()
        self.assertEqual(type(m1), type(m2))
        self.assertEqual(m1, m2)
        self.assertIsNot(m1, m2)

    def test_identity(self):
        self.assertEqual(Matrix33.identity(), [1, 0, 0, 0, 1, 0, 0, 0, 1])

    def test_translate(self):
        self.assertIsNone(Matrix33.translate(None))
        self.assertIsNone(Matrix33.translate([0, 1, 2]))
        self.assertIsNone(Matrix33.translate("Hello world!"))
        self.assertEqual(Matrix33.translate([2]), [1, 0, 0, 0, 1, 0, 2, 2, 1])
        self.assertEqual(Matrix33.translate([2, 3]), [1, 0, 0, 0, 1, 0, 2, 3, 1])

    def test_scale(self):
        self.assertIsNone(Matrix33.scale(None))
        self.assertIsNone(Matrix33.scale([0, 1, 2]))
        self.assertIsNone(Matrix33.scale("Hello world!"))
        self.assertEqual(Matrix33.scale(2), [2, 0, 0, 0, 2, 0, 0, 0, 1])
        self.assertEqual(Matrix33.scale([2, 3]), [2, 0, 0, 0, 3, 0, 0, 0, 1])

    def test_rotate(self):
        self.assertIsNone(Matrix33.rotate(math.nan))
        self.assertEqual(Matrix33.rotate(0), Matrix33())
        self.assertAllAlmostEqual(Matrix33.rotate(1), Matrix33(), places=None, delta=1e-12)
        self.assertAllAlmostEqual(Matrix33.rotate(0.5), [-1, 0, 0, 0, -1, 0, 0, 0, 1], places=None, delta=1e-12)

    def test_mmul(self):
        self.assertEqual(Matrix33() @ Matrix33(), Matrix33())
        self.assertEqual(Matrix33.translate([1, 2]) @ Matrix33(), Matrix33.translate([1, 2]))
        self.assertEqual(Matrix33() @ Matrix33.translate([1, 2]), Matrix33.translate([1, 2]))
        self.assertEqual(Matrix33.translate([1, 2]) @ Matrix33.translate([1, 2]), Matrix33.translate([2, 4]))
        self.assertEqual(Matrix33.translate([1, 2]) @ Matrix33.scale(2), [2, 0, 0, 0, 2, 0, 1, 2, 1])
        self.assertEqual(Matrix33.scale(2) @ Matrix33.translate([1, 2]), [2, 0, 0, 0, 2, 0, 2, 4, 1])

    def test_vmul(self):
        self.assertEqual(Matrix33() @ None, None)
        self.assertEqual(Matrix33() @ [], None)
        self.assertEqual(Matrix33() @ [1], None)
        self.assertEqual(Matrix33() @ [1, 2], [1, 2])
        self.assertEqual(Matrix33() @ [1, 2, 3], [1, 2, 3])
        self.assertEqual(Matrix33() @ [1, 2, 3, 1], None)
        self.assertEqual(Matrix33.translate([1, 2]) @ [0, 0], [1, 2])
        self.assertEqual(Matrix33.translate([1, 2]) @ [-1, -2, 1], [0, 0, 1])
        self.assertEqual(Matrix33.scale(2) @ [-1, -2, 1], [-2, -4, 1])

    def test_inverse(self):
        a = Matrix33.scale([2, 3])
        b = Matrix33.rotate(0.25)
        c = Matrix33.translate([7, 9])
        self.assertAllAlmostEqual(a.inverse() @ a, Matrix33())
        self.assertAllAlmostEqual(b.inverse() @ b, Matrix33())
        self.assertAllAlmostEqual(b.inverse() @ (a.inverse() @ (a @ (b @ c))), c, places=None, delta=1e-12)
        self.assertAllAlmostEqual((((b.inverse() @ a.inverse()) @ a) @ b) @ c, c)
        self.assertAllAlmostEqual(Matrix33([1, 3, -1, 2, 4, 2, 1, 1, -1]).inverse(), [-0.75, 0.25, 1.25, 0.5, 0, -0.5, -0.25, 0.25, -0.25])

    def test_cofactor(self):
        m = Matrix33.scale([2, -3]) @ \
            Matrix33.rotate(1/3) @ \
            Matrix33.translate([7, 9])
        self.assertAllAlmostEqual(m @ m.cofactor().transpose(), Matrix33(m.det()))

    def test_transpose(self):
        self.assertEqual(Matrix33().transpose(), Matrix33())
        self.assertEqual(Matrix33(range(9)).transpose(), [0, 3, 6, 1, 4, 7, 2, 5, 8])

    def test_matrix44(self):
        self.assertEqual(Matrix33().matrix44(), Matrix44())
        self.assertEqual(Matrix33(range(9)).matrix44(), [0, 1, 2, 0, 3, 4, 5, 0, 6, 7, 8, 0, 0, 0, 0, 1])

    def test_repr(self):
        self.assertEqual(repr(Matrix33()), """|   1.000   0.000   0.000 |
|   0.000   1.000   0.000 |
|   0.000   0.000   1.000 |""")



"""
Tests of the model.StateDict class
"""

import pickle
import unittest

from flitter.model import Vector, StateDict, DummyStateDict, null


class TestStateDict(unittest.TestCase):
    def test_construct(self):
        state = StateDict()
        self.assertEqual(len(state), 0)
        state = StateDict({'x': 5})
        self.assertEqual(len(state), 1)
        self.assertEqual(state['x'], Vector(5))

    def test_repr(self):
        state = StateDict({'x': [1, 2, 3], 'y': 'test', 'z': null})
        self.assertEqual(repr(state), "StateDict({'x': 1;2;3, 'y': 'test'})")

    def test_changed(self):
        state = StateDict({'x': 5})
        self.assertFalse(state.changed)
        state['x'] = 6
        self.assertTrue(state.changed)
        self.assertEqual(state.changed_keys, {Vector('x')})
        self.assertEqual(state['x'], Vector(6))
        state['y'] = 7
        self.assertTrue(state.changed)
        self.assertEqual(state.changed_keys, {Vector('x'), Vector('y')})
        self.assertEqual(state['x'], Vector(6))
        self.assertEqual(state['y'], Vector(7))
        state.clear_changed()
        self.assertFalse(state.changed)
        self.assertEqual(state.changed_keys, set())
        del state['x']
        self.assertTrue(state.changed)
        self.assertEqual(state.changed_keys, {Vector('x')})
        self.assertFalse('x' in state)

    def test_set_item(self):
        state = StateDict()
        state['x'] = 5
        self.assertTrue('x' in state)
        self.assertEqual(state.changed_keys, {Vector('x')})
        state.clear_changed()
        state['x'] = null
        self.assertFalse('x' in state)
        self.assertEqual(state.changed_keys, {Vector('x')})

    def test_get_item(self):
        state = StateDict({'x': 5})
        self.assertFalse(state.changed)
        x = state['x']
        self.assertIsInstance(x, Vector)
        self.assertEqual(x, 5)
        y = state['y']
        self.assertIsInstance(y, Vector)
        self.assertEqual(y, null)
        self.assertFalse(state.changed)

    def test_keys(self):
        state = StateDict({'x': 5})
        self.assertEqual(state.keys(), {Vector('x')})
        state['y'] = 7
        self.assertEqual(state.keys(), {Vector('x'), Vector('y')})

    def test_iter(self):
        state = StateDict({'x': 5})
        state['y'] = 7
        i = iter(state)
        self.assertEqual(next(i), Vector('x'))
        self.assertEqual(next(i), Vector('y'))
        with self.assertRaises(StopIteration):
            next(i)

    def test_values(self):
        state = StateDict({'x': 5})
        self.assertEqual(list(state.values()), [Vector(5)])
        state['y'] = 7
        self.assertEqual(list(state.values()), [Vector(5), Vector(7)])

    def test_items(self):
        state = StateDict({'x': 5})
        self.assertEqual(list(state.items()), [(Vector('x'), Vector(5))])
        state['y'] = 7
        self.assertEqual(list(state.items()), [(Vector('x'), Vector(5)), (Vector('y'), Vector(7))])

    def test_clear(self):
        state = StateDict({'x': 5, 'y': 6})
        self.assertEqual(len(state), 2)
        state.clear()
        self.assertEqual(len(state), 0)
        self.assertTrue(state.changed)
        self.assertEqual(state.changed_keys, {Vector('x'), Vector('y')})

    def test_with_keys(self):
        state = StateDict({'x': [1, 2, 3], 'y': 'test', Vector.symbol('z'): Vector.range(10)})
        state2 = state.with_keys([Vector('x'), Vector.symbol('z')])
        self.assertFalse(state.changed)
        self.assertEqual(len(state), 3)
        self.assertFalse(state2.changed)
        self.assertEqual(len(state2), 2)
        self.assertEqual(state2['x'], Vector([1, 2, 3]))
        self.assertEqual(state2[Vector.symbol('z')], Vector.range(10))

    def test_pickling(self):
        state = StateDict({'x': [1, 2, 3], 'y': 'test'})
        state[Vector.symbol('z')] = Vector.range(10)
        self.assertTrue(state.changed)
        state2 = pickle.loads(pickle.dumps(state))
        self.assertFalse(state2.changed)
        self.assertEqual(list(state.items()), list(state2.items()))


class TestDummyState(unittest.TestCase):
    def test_dummy(self):
        state = DummyStateDict()
        self.assertTrue(Vector('x') in state)
        self.assertEqual(state[Vector('x')], null)
        self.assertTrue(Vector(43) in state)
        self.assertEqual(state[Vector(43)], null)
        self.assertTrue(Vector.symbol('z') in state)
        self.assertEqual(state[Vector.symbol('z')], null)



"""
Image window node
"""

from loguru import logger
import PIL.Image

from . import WindowNode
from ...cache import SharedCache
from .target import RenderTarget


class Image(WindowNode):
    def __init__(self, glctx):
        super().__init__(glctx)
        self._filename = None
        self._image = None
        self._target = None

    def release(self):
        self.width = None
        self.height = None
        self._filename = None
        self._image = None
        if self._target is not None:
            self._target.release()
            self._target = None

    @property
    def texture(self):
        return self._target.texture if self._target is not None else None

    @property
    def array(self):
        return self._target.array if self._target is not None else None

    async def update(self, engine, node, references, **kwargs):
        self.node_id = node.get('id', 1, str)
        if self.node_id is not None:
            references[self.node_id] = self
        self.hidden = node.get('hidden', 1, bool, False)
        self.tags = node.tags
        filename = node.get('filename', 1, str)
        if self._image is not None:
            default_size = [self._image.width, self._image.height]
        else:
            default_size = None
        size = node.get('size', 2, int, default_size)
        if filename != self._filename or (size and size != [self.width, self.height]):
            self.release()
            self._filename = filename
        if self._filename is not None:
            image = SharedCache[filename].read_pil_image()
            if image is not self._image:
                self._image = image
                if self._target is not None:
                    self._target.release()
                    self._target = None
                if image is not None:
                    if size and size != [image.width, image.height]:
                        image = image.resize(size, PIL.Image.Resampling.BILINEAR)
                        logger.debug("Resized {} to {}x{}", self._filename, *size)
                    self.width = image.width
                    self.height = image.height
                    image = image.transpose(PIL.Image.FLIP_TOP_BOTTOM).convert('RGBA').convert('RGBa')
                    self._target = RenderTarget.get(self.glctx, self.width, self.height, 8, srgb=True)
                    self._target.texture.write(image.tobytes())

    async def descend(self, engine, node, references, **kwargs):
        # An image is a leaf node from the perspective of the OpenGL world
        pass

    def similar_to(self, node):
        return super().similar_to(node) and (node.get('filename', 1, str) == self._filename or node.get('id', 1, str) == self.node_id)



"""
Flitter engine functional tests
"""

import asyncio
from pathlib import Path
import sys
import sysconfig
import tempfile
import unittest

import PIL.Image
import PIL.ImageChops
import PIL.ImageStat

from flitter.engine.control import EngineController


def image_diff(ref, img):
    """Return how dissimilar two images are as a score between 0 (identical) and 1 (completely different)"""
    return PIL.ImageStat.Stat(PIL.ImageChops.difference(ref, img).convert('L')).sum[0] / (img.width * img.height * 255)


TEST_IMAGES_DIR = Path(__file__).parent.parent / f'build/test_images.{sysconfig.get_platform()}-{sys.implementation.cache_tag}'
TEST_IMAGES_DIR.mkdir(mode=0o775, parents=True, exist_ok=True)


class TestRendering(unittest.TestCase):
    """
    Some simple rendering functional tests

    Scripts will be executed for a single frame and a `SIZE` image should be
    saved to the filename `OUTPUT`.
    """

    SIZE = (200, 100)

    def setUp(self):
        self.script_path = Path(tempfile.mktemp('.fl'))
        self.input_path = Path(tempfile.mktemp('.png'))
        self.output_path = Path(tempfile.mktemp('.png'))
        self.controller = EngineController(realtime=False, offscreen=True, target_fps=1, run_time=1,
                                           defined_names={'SIZE': self.SIZE,
                                                          'INPUT': str(self.input_path),
                                                          'OUTPUT': str(self.output_path)})

    def tearDown(self):
        if self.script_path.exists():
            self.script_path.unlink()
        if self.input_path.exists():
            self.input_path.unlink()
        if self.output_path.exists():
            self.output_path.unlink()

    def test_simple_canvas(self):
        """Render a simple green rectangle filling the viewport"""
        self.script_path.write_text("""
!window size=SIZE
    !record filename=OUTPUT
        !canvas
            !path
                !rect size=SIZE
                !fill color=0;1;0
""", encoding='utf8', newline='\n')
        self.controller.load_page(self.script_path)
        asyncio.run(self.controller.run())
        img = PIL.Image.open(self.output_path)
        self.assertEqual(img.size, self.SIZE)
        self.assertEqual(img.tobytes(), b'\x00\xff\x00' * self.SIZE[0] * self.SIZE[1])

    def test_simple_canvas3d(self):
        """Render a simple green emissive box filling the viewport"""
        self.script_path.write_text("""
!window size=SIZE
    !record filename=OUTPUT
        !canvas3d orthographic=true
            !box size=SIZE*(1;1;0) emissive=0;1;0
""", encoding='utf8', newline='\n')
        self.controller.load_page(self.script_path)
        asyncio.run(self.controller.run())
        img = PIL.Image.open(self.output_path)
        self.assertEqual(img.size, self.SIZE)
        self.assertEqual(img.tobytes(), b'\x00\xff\x00' * self.SIZE[0] * self.SIZE[1])

    def test_simple_shader(self):
        """Render a simple shader that returns green for all fragments"""
        self.script_path.write_text("""
!window size=SIZE
    !record filename=OUTPUT
        !shader fragment='''#version 330
                            in vec2 coord;
                            out vec4 color;
                            void main() {
                                color = vec4(0.0, 1.0, 0.0, 1.0);
                            }'''
""", encoding='utf8', newline='\n')
        self.controller.load_page(self.script_path)
        asyncio.run(self.controller.run())
        img = PIL.Image.open(self.output_path)
        self.assertEqual(img.size, self.SIZE)
        self.assertEqual(img.tobytes(), b'\x00\xff\x00' * self.SIZE[0] * self.SIZE[1])

    def test_simple_image(self):
        """Render an image node from a simple temporary image"""
        PIL.Image.new('RGB', self.SIZE, (0, 255, 0)).save(self.input_path, 'PNG')
        self.script_path.write_text("""
!window size=SIZE
    !record filename=OUTPUT
        !image filename=INPUT
""", encoding='utf8', newline='\n')
        self.controller.load_page(self.script_path)
        asyncio.run(self.controller.run())
        img = PIL.Image.open(self.output_path)
        self.assertEqual(img.size, self.SIZE)
        self.assertEqual(img.tobytes(), b'\x00\xff\x00' * self.SIZE[0] * self.SIZE[1])


class ScriptTest(unittest.TestCase):
    def assertScriptOutputMatchesImage(self, script, suffix='.png', target_fps=1, run_time=1, **kwargs):
        output_path = TEST_IMAGES_DIR / script.with_suffix(suffix).name
        if output_path.exists():
            output_path.unlink()
        reference = PIL.Image.open(script.with_suffix('.png'))
        controller = EngineController(realtime=False, offscreen=True, defined_names={'OUTPUT': str(output_path)},
                                      target_fps=target_fps, run_time=run_time, **kwargs)
        controller.load_page(script)
        asyncio.run(controller.run())
        output = PIL.Image.open(output_path)
        self.assertEqual(reference.size, output.size, msg="Size mismatch")
        self.assertLess(image_diff(reference, output), 0.004, msg="Output differs from reference image")

    def assertAllScriptOutputsMatchesImages(self, scripts, suffix='.png', target_fps=1, run_time=1, **kwargs):
        for i, script in enumerate(scripts):
            with self.subTest(script=script, **kwargs):
                self.assertScriptOutputMatchesImage(script, suffix=suffix, target_fps=target_fps, run_time=run_time, **kwargs)


class TestDocumentationDiagrams(ScriptTest):
    """
    Recreate the documentation diagrams and check them against the pre-calculated ones.
    """

    DIAGRAMS = Path(__file__).parent.parent / 'docs/diagrams'

    def test_box_uvmap(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'box_uvmap.fl')

    def test_dummyshader(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'dummyshader.fl')

    def test_easings(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'easings.fl')

    def test_petri(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'petri.fl', target_fps=10, run_time=10)

    def test_pseudorandoms(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'pseudorandoms.fl')

    def test_spheroidbox(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'spheroidbox.fl')

    def test_torus(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'torus.fl')

    def test_waveforms(self):
        self.assertScriptOutputMatchesImage(self.DIAGRAMS / 'waveforms.fl')


class TestDocumentationTutorial(ScriptTest):
    """
    Recreate the tutorial images and check them against the pre-calculated ones.
    """

    TUTORIAL_IMAGES = Path(__file__).parent.parent / 'docs/tutorial_images'

    def test_tutorial1(self):
        self.assertScriptOutputMatchesImage(self.TUTORIAL_IMAGES / 'tutorial1.fl')

    def test_tutorial2(self):
        self.assertScriptOutputMatchesImage(self.TUTORIAL_IMAGES / 'tutorial2.fl')

    def test_tutorial3(self):
        self.assertScriptOutputMatchesImage(self.TUTORIAL_IMAGES / 'tutorial3.fl')

    def test_tutorial4(self):
        self.assertScriptOutputMatchesImage(self.TUTORIAL_IMAGES / 'tutorial4.fl')

    def test_tutorial5(self):
        self.assertScriptOutputMatchesImage(self.TUTORIAL_IMAGES / 'tutorial5.fl')

    def test_tutorial6(self):
        self.assertScriptOutputMatchesImage(self.TUTORIAL_IMAGES / 'tutorial6.fl')


class TestExamples(ScriptTest):
    """
    Recreate the examples and check them against the pre-calculated ones.
    """

    EXAMPLES = Path(__file__).parent.parent / 'examples'

    def test_bauble(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'bauble.fl')

    def test_bounce(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'bounce.fl', target_fps=10)

    def test_canvas3d(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'canvas3d.fl')

    def test_dots(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'dots.fl', target_fps=10)

    def test_hoops(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'hoops.fl', target_fps=10)

    def test_linear(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'linear.fl')

    def test_linelight(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'linelight.fl')

    def test_oklch(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'oklch.fl')

    def test_physics(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'physics.fl', target_fps=10)

    def test_sdf(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'sdf.fl')

    def test_smoke(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'smoke.fl', target_fps=10)

    def test_solidgeometry(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'solidgeometry.fl')

    def test_sphere(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'sphere.fl')

    def test_teaset(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'teaset.fl')

    def test_textures(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'textures.fl', target_fps=2)

    def test_translucency(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'translucency.fl')

    def test_video(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'video.fl')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_bauble_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'bauble.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_bounce_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'bounce.fl', target_fps=10, opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_canvas3d_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'canvas3d.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_linelight_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'linelight.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_oklch_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'oklch.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_physics_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'physics.fl', target_fps=10, opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_sdf_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'sdf.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_solidgeometry_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'solidgeometry.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_teaset_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'teaset.fl', opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_textures_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'textures.fl', target_fps=2, opengl_es=True, suffix='.es.png')

    @unittest.skipIf(sys.platform != 'linux', 'OpenGL ES only available on Linux')
    def test_translucency_opengl_es(self):
        self.assertScriptOutputMatchesImage(self.EXAMPLES / 'translucency.fl', opengl_es=True, suffix='.es.png')
